@@ -7,21 +7,9 @@ no if $] >= 5.017011, warnings => 'experimental::smartmatch';
 use utf8;
 
 use Data::Dumper;
-use JSON::XS qw();
+use JSON qw();
 
-our $VERSION = '0.18';
-
-our %CAN;
-
-BEGIN {
-    $CAN{'highlight'} = eval {
-        # FIXME: probably will not work if not unix os
-        qx(which highlight 2>/dev/null) and $? == 0
-    };
-    # $CAN{'highlight'}->{'error'} = $@;
-    $CAN{'ddp'} = eval { require Data::Printer };
-    $CAN{'_error'}->{'ddp'} = $@;
-}
+our $VERSION = '0.19';
 
 sub MAX_RECURSION () {300}
 
@@ -51,7 +39,7 @@ sub read_next_json_file () {
     local $SIG{__WARN__} = \&warn_without_line;
     while (<>) {
         utf8::encode($_) if utf8::is_utf8($_);
-        $_ = eval { JSON::XS::decode_json($_) };
+        $_ = eval { JSON::decode_json($_) };
         warn "Failed to parse $ARGV: $@" if $@;
         next unless ref $_;
         return $_;
@@ -307,108 +295,21 @@ sub modify_data {
     }
 }
 
-sub highlight ($$) {
-    my ($string, $lang) = @_;
-
-    return $string unless $CAN{'highlight'};
-
-    # IPC::Open2 hangs on big data so we will do like this
-    my $pid = open(my $hl_out, '-|');
-    if (not defined $pid) {
-        warn "highlight failed: $!\n";
-        return $string;
-    }
-    my $utf8 = utf8::is_utf8($string);
-
-    unless ($pid) {
-        open(my $hl_in, '|-', 'highlight', '-O', 'ansi', '-S', $lang)
-          or die "$!\n";
-        utf8::encode($string) if utf8::is_utf8($string);
-        print $hl_in $string;
-        close $hl_in;
-        exit 0;
-    }
-
-    local $/;
-    my $highlighted = <$hl_out>;
-    close $hl_out;    # may be waitpid($pid, 0); ??
-    return $string unless $? == 0;
-    utf8::decode($highlighted) if $utf8 and not utf8::is_utf8($highlighted);
-
-    return $highlighted || $string;
-}
-
-our %OUT = (
-    'ddp' => sub {
-        my ($data, $args) = @_;
-        my $ddp = Data::Printer::np($data,
-            colored   => $args->{'color'},
-            max_depth => $args->{'max_depth'},
-        );
-        utf8::encode($ddp) if utf8::is_utf8($ddp);
-        say $ddp;
-    },
-    'json' => sub {
-        my ($data, $args) = @_;
-        local $SIG{__WARN__} = \&warn_without_line;
-
-        if ($args->{'max_depth'}) {
-            my $VAR1;
-            eval Data::Dumper->new([$data])->Maxdepth($args->{'max_depth'})->Dump();
-            if ($@ or not defined $VAR1) {
-                warn "max_depth failed: $@";
-            }
-            else {
-                $data = $VAR1;
-            }
-        }
-
-        my $json = eval {
-            JSON::XS
-              ->new
-              ->pretty(int not $args->{'minimal'})
-              ->allow_nonref
-              ->canonical
-              ->encode($data)
-        };
-        warn($@), return if $@;
-        $json = highlight($json, 'js') if $args->{'color'};
-        utf8::encode($json) if utf8::is_utf8($json);
-        print $json;
-    },
-    'perl' => sub {
-        my ($data, $args) = @_;
-        local $SIG{__WARN__} = \&warn_without_line;
-        local $Data::Dumper::Useperl = 1;
-        no warnings 'redefine';
-        local *Data::Dumper::qquote = sub {
-            my $d = shift;
-            utf8::encode $d if utf8::is_utf8($d);
-            return "'$d'";
-        };
-        my $perl = eval {
-            Data::Dumper->new([$data])
-              ->Indent(int not $args->{'minimal'})
-              ->Pair($args->{'minimal'} ? '=>' : ' => ')
-              ->Terse(1)
-              ->Sortkeys(int not $args->{'minimal'})
-              ->Useqq(1)
-              ->Maxdepth($args->{'max_depth'})
-              ->Dump()
-        };
-        warn($@), return if $@;
-        $perl = highlight($perl, 'perl') if $args->{'color'};
-        utf8::encode($perl) if utf8::is_utf8($perl);
-        print $perl;
-    },
-);
-
-sub validate_output ($) {
+sub output ($) {
     my $output = shift;
-    die 'Output must be one of ' . join(', ' => map {"'$_'"} sort keys %OUT) . "\n"
-      unless $OUT{$output};
-    die "Can't require Data::Printer : $CAN{'_error'}->{'ddp'}\n"
-      if not $CAN{'ddp'} and $output eq 'ddp';
+    state $supported = {
+        'ddp'  => 'App::yajg::Output::DDP',
+        'json' => 'App::yajg::Output::Json',
+        'perl' => 'App::yajg::Output::Perl',
+        'yaml' => 'App::yajg::Output::Yaml',
+    };
+    die 'Output must be one of ' . join(', ' => map {"'$_'"} sort keys %$supported) . "\n"
+      unless $supported->{$output};
+
+    eval "require $supported->{$output}";
+    die "Can't init output $output: $@" if $@;
+
+    return $supported->{$output}->new;
 }
 
 1;
@@ -419,7 +320,7 @@ __END__
 
 =head1 NAME
 
-B<App::yajg> - yet another json grep.
+App::yajg - yet another json grep
 
 =head1 SYNOPSIS
 
@@ -528,12 +429,14 @@ Select the output type. Supported types are:
 
 =over 2
 
-=item * json (via L<JSON::XS>)
+=item * json (via L<JSON>)
 
 =item * perl (via L<Data::Dumper>)
 
 =item * ddp (via L<Data::Printer>)
 
+=item * yaml (via L<YAML>)
+
 =back
 
 If the L<Data::Printer> installed than the default value will be ddp otherwise
@@ -555,8 +458,8 @@ Convert boolean types to defined format:
 
 Maby usefull because by default all C<true> is ref to
 C<Types::Serialiser::true> and all C<false> is ref to
-C<Types::Serialiser::false> and the output in the B<perl> or the B<ddp> format
-looks very ugly.
+C<Types::Serialiser::false> and the output in the B<perl>, the B<ddp> or the
+B<yaml> format looks very ugly.
 
 =item B<-c, --color, --no-color>
 
@@ -681,6 +584,12 @@ F<exaple.json>
      ]
  }
 
+ $ yajg -p id -o yaml exaple.json
+ ---
+ array:
+   - id: test
+   - id: test_2
+
 =head2 Value grep
 
  yajg -P '^1$' exaple.json
@@ -853,12 +762,14 @@ Regexp example:
 
 =over 4
 
-=item L<JSON::XS>
+=item L<JSON>
 
 =item L<Data::Dumper>
 
 =item L<Data::Printer>
 
+=item L<YAML>
+
 =item L<highlight(1)>
 
 =back
